@@ -1,7 +1,9 @@
 library(RxODE)
 rxPermissive({
+
     rxClean()
     for (meth in c("liblsoda", "lsoda")){ ## Dop is very close but doesn't match precisely.
+
         context(sprintf("Simple test for time-varying covariates (%s)", meth))
 
         ode <- RxODE({
@@ -12,12 +14,17 @@ rxPermissive({
             printf("%.10f,%.10f\n",t,c);
         })
 
-        et <- eventTable()   # default time units
+        et <- eventTable(time.units="hr")   # default time units
         et$add.sampling(seq(from=0, to=100, by=0.01))
 
-        cov <- data.frame(c=et$get.sampling()$time+1);
+        cov <- data.frame(c=et$get.EventTable()$time+units::set_units(1,h));
+
+        et0 <- et;
+
+        et <- cbind(et,cov)
+
+        cov.lin <- approxfun(et$time, et$c, yleft=et$c[1], yright=et$c[length(cov$c)]);
 
-        cov.lin <- approxfun(et$get.sampling()$time, cov$c, yleft=cov$c[1], yright=cov$c[length(cov$c)]);
 
         sink("temp.csv");
         cat("t,c\n");
@@ -25,7 +32,7 @@ rxPermissive({
                        params = c(a=-8/3, b=-10),
                        events = et,
                        inits = c(X=1, Y=1, Z=1),
-                       covs = cov, add.cov=TRUE,
+                       add.cov=TRUE,
                        covsInterpolation="linear",
                        method=meth);
         sink();
@@ -49,7 +56,6 @@ rxPermissive({
                        params = c(a=-8/3, b=-10),
                        events = et,
                        inits = c(X=1, Y=1, Z=1),
-                       covs = cov,
                        covs_interpolation="NOCB", add.cov=TRUE,
                        method=meth);
         sink()
@@ -72,7 +78,6 @@ rxPermissive({
                        params = c(a=-8/3, b=-10),
                        events = et,
                        inits = c(X=1, Y=1, Z=1),
-                       covs = cov,
                        covs_interpolation="midpoint", add.cov=TRUE,
                        method=meth);
         sink()
@@ -96,7 +101,6 @@ rxPermissive({
                        params = c(a=-8/3, b=-10),
                        events = et,
                        inits = c(X=1, Y=1, Z=1),
-                       covs = cov,
                        covs_interpolation="constant", add.cov=TRUE,
                        method=meth);
         sink()
@@ -132,39 +136,43 @@ rxPermissive({
             expect_false(isTRUE(all.equal(out,out1)));
         })
 
-        cov <- data.frame(c=et$get.sampling()$time+1,a=-et$get.sampling()$time/100);
+        cov <- data.frame(c=et0$get.EventTable()$time+units::set_units(1,hr),
+                          a=-et0$get.EventTable()$time/units::set_units(100,hr));
+
+        et <- cbind(et0,cov)
 
         sink("temp")
         out <- rxSolve(ode,
                        params = c(a=-8/3, b=-10),
                        events = et,
                        inits = c(X=1, Y=1, Z=1),
-                       covs = cov, add.cov=TRUE,
+                       add.cov=TRUE,
                        method=meth)
 
         out3 <- rxSolve(ode,
                         params = c(a=-8/3, b=-10),
                         events = et,
                         inits = c(X=1, Y=1, Z=1),
-                        covs = cov, add.cov=TRUE,
+                        add.cov=TRUE,
                         method=meth)
         sink();
         unlink("temp");
 
         test_that("time varying covariates output covariate in data frame",{
-            expect_equal(cov$c,out$c);
-            expect_equal(cov$a,out$a);
+            expect_equal(cov$c[et0$get.obs.rec()],out$c);
+            expect_equal(cov$a[et0$get.obs.rec()],out$a);
         })
 
 
-        cov <- data.frame(c=et$get.sampling()$time+1);
+        cov <- data.frame(c=et0$get.EventTable()$time+units::set_units(1,hr));
+        et <- cbind(et0, cov);
 
         sink("temp");
         out2 <- rxSolve(ode,
                         params = c(a=-8/3, b=-10),
                         events = et,
                         inits = c(X=1, Y=1, Z=1),
-                        covs = cov, add.cov=TRUE,
+                        add.cov=TRUE,
                         method=meth)
         sink();
         unlink("temp");
@@ -207,6 +215,7 @@ rxPermissive({
                         class = "data.frame",
                         row.names = c(NA,  -6L),
                         .Names = c("TIME", "AMT", "V2I", "V1I", "CLI", "EVID"))
+
         mod1 = RxODE({
             d/dt(A_centr)=-A_centr*(CLI/V1I+204/V1I)+204*A_periph/V2I;
             d/dt(A_periph)=204*A_centr/V1I-204*A_periph/V2I;
@@ -221,6 +230,7 @@ rxPermissive({
             d/dt(A_tr3)=4*A_tr2*exp(-ETA[2]-THETA[2])-4*A_tr3*exp(-ETA[2]-THETA[2]);
             A_tr3(0)=exp(ETA[1]+THETA[1]);
         })
+
         tmp <- rxSolve(mod1, d3, structure(c(2.02103, 4.839305, 3.518676, -1.391113, 0.108127023, -0.064170725, 0.087765769),
                                            .Names=c(sprintf("THETA[%d]", 1:4), sprintf("ETA[%d]", 1:3))), add.cov=TRUE,
                        method=meth)
@@ -252,6 +262,44 @@ rxPermissive({
             expect_equal(tmp %>% select(CLI, V1I, V2I) %>% as.data.frame, d3 %>% filter(EVID == 0) %>% select(CLI,V1I,V2I) %>% as.data.frame)
             expect_equal(names(tmp$params)[-1], mod1$params[-(1:3)])
         })
+
+        ## Now check missing covariate values.
+
+        d3na <- structure(list(ID = c(1L, 1L, 1L, 1L, 1L, 1L, 2L, 2L, 2L, 2L),
+                             TIME = c(0, 0, 2.99270072992701, 192, 336, 456, 0, 0, 3.07272727272727, 432),
+                             AMT = c(137L, 0L, -137L, 0L, 0L, 0L, 110L, 0L, -110L, 0L),
+                             V2I = c(909L, NA_integer_, 909L, 909L, 909L, 909L, 942L, 942L, 942L, 942L),
+                             V1I = c(545L, 545L, 545L, 545L, 545L, 545L, 306L, 306L, 306L, NA_integer_),
+                             CLI = c(471L, 471L, 471L, 471L, NA_integer_, 471L, 405L, 405L, 405L, 405L),
+                             EVID = c(10101L, 0L, 10101L, 0L, 0L, 0L, 10101L, 0L, 10101L, 0L)),
+                        class = "data.frame", row.names = c(NA,  -10L),
+                        .Names = c("ID", "TIME", "AMT", "V2I", "V1I", "CLI", "EVID"))
+
+        tmp <- rxSolve(mod1, d3na, par2, add.cov=TRUE, cores=2, method=meth)
+
+        tmp2 <- rxSolve(mod1, d3, par2, add.cov=TRUE, cores=2, method=meth)
+
+        context(sprintf("Test NA extrapolation for %s solving", meth))
+        test_that("NA solve is the same", {
+            for (i in c("id", "time", "A_centr", "A_periph", "A_circ", "A_prol", "A_tr1", "A_tr2", "A_tr3")){
+                expect_equal(tmp[[i]],tmp2[[i]])
+            }
+        })
+
+        d3na <- structure(list(ID = c(1L, 1L, 1L, 1L, 1L, 1L, 2L, 2L, 2L, 2L),
+                               TIME = c(0, 0, 2.99270072992701, 192, 336, 456, 0, 0, 3.07272727272727, 432),
+                               AMT = c(137L, 0L, -137L, 0L, 0L, 0L, 110L, 0L, -110L, 0L),
+                               V2I = c(909L, NA_integer_, 909L, 909L, 909L, 909L, 942L, 942L, 942L, 942L),
+                               V1I = c(545L, 545L, 545L, 545L, 545L, 545L, NA_integer_, NA_integer_, NA_integer_, NA_integer_),
+                               CLI = c(471L, 471L, 471L, 471L, NA_integer_, 471L, 405L, 405L, 405L, 405L),
+                               EVID = c(10101L, 0L, 10101L, 0L, 0L, 0L, 10101L, 0L, 10101L, 0L)),
+                          class = "data.frame", row.names = c(NA,  -10L),
+                          .Names = c("ID", "TIME", "AMT", "V2I", "V1I", "CLI", "EVID"))
+
+        test_that("All covariates are NA give a warning",{
+                    expect_warning(rxSolve(mod1, d3na, par2, add.cov=TRUE, cores=2, method=meth),"One or more covariates were all NA for subject id=2")
+        })
+
     }
     ## devtools::install();library(RxODE);rxTest("cov")
     rxClean()
